pub struct TempoTransactionPool<Client> {
protocol_pool: Pool<TransactionValidationTaskExecutor<TempoTransactionValidator<Client>>, TempoTipOrdering<TempoPooledTransaction>, InMemoryBlobStore>,
aa_2d_pool: Arc<RwLock<AA2dPool>>,
}Expand description
Tempo transaction pool that routes based on nonce_key
Fields
protocol_pool: Pool<TransactionValidationTaskExecutor<TempoTransactionValidator<Client>>, TempoTipOrdering<TempoPooledTransaction>, InMemoryBlobStore>Vanilla pool for all standard transactions and AA transactions with regular nonce.
aa_2d_pool: Arc<RwLock<AA2dPool>>Minimal pool for 2D nonces (nonce_key > 0)
Implementations
Sourceimpl<Client> TempoTransactionPool<Client>where
Client: StateProviderFactory + ChainSpecProvider<ChainSpec = TempoChainSpec> + 'static,
impl<Client> TempoTransactionPool<Client>where
Client: StateProviderFactory + ChainSpecProvider<ChainSpec = TempoChainSpec> + 'static,
pub fn new( protocol_pool: Pool<TransactionValidationTaskExecutor<TempoTransactionValidator<Client>>, TempoTipOrdering<TempoPooledTransaction>, InMemoryBlobStore>, aa_2d_pool: AA2dPool, ) -> Self
Sourceimpl<Client> TempoTransactionPool<Client>where
Client: StateProviderFactory + ChainSpecProvider<ChainSpec = TempoChainSpec> + 'static,
impl<Client> TempoTransactionPool<Client>where
Client: StateProviderFactory + ChainSpecProvider<ChainSpec = TempoChainSpec> + 'static,
Sourcepub fn amm_liquidity_cache(&self) -> AmmLiquidityCache
pub fn amm_liquidity_cache(&self) -> AmmLiquidityCache
Obtains a clone of the shared AmmLiquidityCache.
Sourcepub(crate) fn notify_aa_pool_on_state_updates(
&self,
state: &AddressMap<BundleAccount>,
) -> Vec<Arc<ValidPoolTransaction<TempoPooledTransaction>>>
pub(crate) fn notify_aa_pool_on_state_updates( &self, state: &AddressMap<BundleAccount>, ) -> Vec<Arc<ValidPoolTransaction<TempoPooledTransaction>>>
Updates the 2d nonce pool with the given state changes.
Returns mined AA transactions.
Sourcepub fn evict_invalidated_transactions(
&self,
updates: &TempoPoolUpdates,
) -> Vec<Arc<ValidPoolTransaction<TempoPooledTransaction>>>
pub fn evict_invalidated_transactions( &self, updates: &TempoPoolUpdates, ) -> Vec<Arc<ValidPoolTransaction<TempoPooledTransaction>>>
Evicts transactions that are no longer valid due to on-chain events.
This performs a single scan of all pooled transactions and checks for:
- Revoked keychain keys: AA transactions signed with keys that have been revoked
- Spending limit updates: AA transactions signed with keys whose spending limit
changed for a token matching the transaction’s fee token
2b. Spending limit spends: AA transactions whose remaining spending limit (re-read
from state) is now insufficient after included keychain txs decremented it
2c. Key-authorization witness burns: AA transactions with a witness-bearing
inline key authorization whose
(account, witness)has been manually burned - Validator token changes: Transactions that would fail due to insufficient liquidity in the new (user_token, validator_token) AMM pool
- Fee payer balance changes: Transactions whose fee payer no longer has enough balance in the resolved fee token after a TIP20 transfer
All checks are combined into one scan to avoid iterating the pool multiple times per block.
Sourcepub(crate) fn evict_invalidated_transactions_from<'a>(
&self,
updates: &TempoPoolUpdates,
transactions: impl IntoIterator<Item = &'a Arc<ValidPoolTransaction<TempoPooledTransaction>>>,
) -> Vec<Arc<ValidPoolTransaction<TempoPooledTransaction>>>
pub(crate) fn evict_invalidated_transactions_from<'a>( &self, updates: &TempoPoolUpdates, transactions: impl IntoIterator<Item = &'a Arc<ValidPoolTransaction<TempoPooledTransaction>>>, ) -> Vec<Arc<ValidPoolTransaction<TempoPooledTransaction>>>
See Self::evict_invalidated_transactions; returns the removed transactions so
the caller controls when they are dropped.
Sourcefn add_validated_transaction(
&self,
origin: TransactionOrigin,
transaction: TransactionValidationOutcome<TempoPooledTransaction>,
) -> PoolResult<AddedTransactionOutcome>
fn add_validated_transaction( &self, origin: TransactionOrigin, transaction: TransactionValidationOutcome<TempoPooledTransaction>, ) -> PoolResult<AddedTransactionOutcome>
Adds a validated transaction to the subpool derived from its type and nonce key.
TempoPooledTransaction::is_aa_2d routes AA transactions with non-zero
nonce keys, including expiring nonces, to the 2D nonce pool. Everything else
stays in the protocol pool.