pub struct TempoTransactionPool<Client> {
protocol_pool: Pool<TransactionValidationTaskExecutor<TempoTransactionValidator<Client>>, CoinbaseTipOrdering<TempoPooledTransaction>, InMemoryBlobStore>,
aa_2d_pool: Arc<RwLock<AA2dPool>>,
}Expand description
Tempo transaction pool that routes based on nonce_key
Fields
protocol_pool: Pool<TransactionValidationTaskExecutor<TempoTransactionValidator<Client>>, CoinbaseTipOrdering<TempoPooledTransaction>, InMemoryBlobStore>Vanilla pool for all standard transactions and AA transactions with regular nonce.
aa_2d_pool: Arc<RwLock<AA2dPool>>Minimal pool for 2D nonces (nonce_key > 0)
Implementations
Sourceimpl<Client> TempoTransactionPool<Client>
impl<Client> TempoTransactionPool<Client>
pub fn new( protocol_pool: Pool<TransactionValidationTaskExecutor<TempoTransactionValidator<Client>>, CoinbaseTipOrdering<TempoPooledTransaction>, InMemoryBlobStore>, aa_2d_pool: AA2dPool, ) -> Self
Sourceimpl<Client> TempoTransactionPool<Client>where
Client: StateProviderFactory + ChainSpecProvider<ChainSpec = TempoChainSpec> + 'static,
impl<Client> TempoTransactionPool<Client>where
Client: StateProviderFactory + ChainSpecProvider<ChainSpec = TempoChainSpec> + 'static,
Sourcepub fn amm_liquidity_cache(&self) -> AmmLiquidityCache
pub fn amm_liquidity_cache(&self) -> AmmLiquidityCache
Obtains a clone of the shared AmmLiquidityCache.
Sourcepub(crate) fn notify_aa_pool_on_state_updates(
&self,
state: &AddressMap<BundleAccount>,
)
pub(crate) fn notify_aa_pool_on_state_updates( &self, state: &AddressMap<BundleAccount>, )
Updates the 2d nonce pool with the given state changes.
Sourcepub(crate) fn reset_2d_nonces_from_state(
&self,
seq_ids: Vec<AASequenceId>,
block_hash: B256,
) -> Result<(), ProviderError>
pub(crate) fn reset_2d_nonces_from_state( &self, seq_ids: Vec<AASequenceId>, block_hash: B256, ) -> Result<(), ProviderError>
Resets the nonce state for the given 2D nonce sequence IDs by reading from a specific block’s state. Used during reorgs to correct the pool’s nonce tracking for slots that were modified in the old chain but not in the new chain.
Sourcepub(crate) fn remove_included_expiring_nonce_txs<'a>(
&self,
tx_hashes: impl Iterator<Item = &'a TxHash>,
)
pub(crate) fn remove_included_expiring_nonce_txs<'a>( &self, tx_hashes: impl Iterator<Item = &'a TxHash>, )
Removes expiring nonce transactions that were included in a block.
This is called with the transaction hashes from mined blocks to clean up expiring nonce transactions on inclusion, rather than waiting for expiry.
Sourcepub fn evict_invalidated_transactions(
&self,
updates: &TempoPoolUpdates,
) -> Vec<TxHash> ⓘ
pub fn evict_invalidated_transactions( &self, updates: &TempoPoolUpdates, ) -> Vec<TxHash> ⓘ
Evicts transactions that are no longer valid due to on-chain events.
This performs a single scan of all pooled transactions and checks for:
- Revoked keychain keys: AA transactions signed with keys that have been revoked
- Spending limit updates: AA transactions signed with keys whose spending limit changed for a token matching the transaction’s fee token 2b. Spending limit spends: AA transactions whose remaining spending limit (re-read from state) is now insufficient after included keychain txs decremented it
- Validator token changes: Transactions that would fail due to insufficient liquidity in the new (user_token, validator_token) AMM pool
All checks are combined into one scan to avoid iterating the pool multiple times per block.
fn add_validated_transaction( &self, origin: TransactionOrigin, transaction: TransactionValidationOutcome<TempoPooledTransaction>, ) -> PoolResult<AddedTransactionOutcome>